#include "sdk_common.h"
#if NRF_MODULE_ENABLED(NRF_CLI)
#include <stdint.h>
#include <string.h>

#include "bsp.h"
#include "ble.h"
#include "nrf_log.h"
#include "nrf_cli.h"
#include "nrf_cli_uart.h"
#include "nrf_cli_types.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_sdh.h"
#include "nrf_delay.h"
#include "cli.h"
#include "FreeRTOS.h"
#include "task.h"


#define CLI_EXAMPLE_LOG_QUEUE_SIZE  (15)
#define CLI_STACK_SIZE 512
#define MAX_GPIO_NUM 47
#define CLI_PRIORITY 1

#if NRF_CLI_UART_ENABLED 
NRF_CLI_UART_DEF(m_cli_uart_transport, APP_UART_DRIVER_INSTANCE, 64, 16);
NRF_CLI_DEF(m_cli_uart,"uart_cli:~$ ",&m_cli_uart_transport.transport,'\r',CLI_EXAMPLE_LOG_QUEUE_SIZE);
#endif

#if NRF_CLI_RTT_ENABLED  
NRF_CLI_RTT_DEF(m_cli_rtt_transport);
NRF_CLI_DEF(m_cli_rtt,"rtt_cli:~$ ",&m_cli_rtt_transport.transport,'\n',CLI_EXAMPLE_LOG_QUEUE_SIZE);
#endif

static TaskHandle_t m_cli_thread;      /**< Logger thread. */

static void cli_thread(void * arg);
static void cli_start(void);
extern void cli_process(void);
void cli_task_create(void);
static void cli_uart_setup(void);
#define GPIO_HELP "control specific gpio pin\r\n" \
                  "usage: gpio get|set|reset|cfg_read|in_cfg|out_cfg|toggle [gpio #]\r\n"

void cli_unknown_param_help(nrf_cli_t const *p_cli, char const *p_param, char const *p_cmd) {
    nrf_cli_fprintf(p_cli, NRF_CLI_ERROR, "%s: unknown parameter '%s'\r\nTry '%s -h' for help.\r\n", p_cmd, p_param, p_cmd);
}

void cli_missing_param_help(nrf_cli_t const *p_cli, char const *p_cmd) {
    nrf_cli_fprintf(p_cli, NRF_CLI_ERROR, "%s: missing parameter.\r\nTry '%s -h' for help.\r\n",p_cmd, p_cmd);
}

static void cli_thread(void * arg)
{
    UNUSED_PARAMETER(arg);
    cli_start();

    for (;;)
    {
        vTaskDelay(pdMS_TO_TICKS(100));
        cli_process();
    }
}

static void cli_start(void)
{
    ret_code_t rc;
#if NRF_CLI_UART_ENABLED 
    rc = nrf_cli_start(&m_cli_uart);
    APP_ERROR_CHECK(rc);
#endif
#if NRF_CLI_RTT_ENABLED 
    rc = nrf_cli_start(&m_cli_rtt);
    APP_ERROR_CHECK(rc);
#endif 
}

void cli_process(void) 
{
#if NRF_CLI_UART_ENABLED
    nrf_cli_process(&m_cli_uart);
#endif
#if NRF_CLI_RTT_ENABLED  
    nrf_cli_process(&m_cli_rtt);
#endif
}

void cli_task_create(void)
{
    if (pdPASS != xTaskCreate(cli_thread,
                              "CLI",
                              CLI_STACK_SIZE,
                              NULL,
                              CLI_PRIORITY,
                              &m_cli_thread))
    {
        APP_ERROR_HANDLER(NRF_ERROR_NO_MEM);
    }
}

void cli_init(void) 
{
    ret_code_t rc;
#if  NRF_CLI_UART_ENABLED
    nrf_drv_uart_config_t uart_config = NRF_DRV_UART_DEFAULT_CONFIG;
    uart_config.pseltxd = TX_PIN_NUMBER;
    uart_config.pselrxd = RX_PIN_NUMBER;
    uart_config.hwfc = NRF_UART_HWFC_DISABLED;
    rc = nrf_cli_init(&m_cli_uart, &uart_config, true, true, NRF_LOG_SEVERITY_INFO);
    APP_ERROR_CHECK(rc);
#endif

#if NRF_CLI_RTT_ENABLED 
    rc = nrf_cli_init(&m_cli_rtt, NULL, true, true, NRF_LOG_SEVERITY_INFO);
    APP_ERROR_CHECK(rc);
#endif 
}

static void gpio_cmd_get(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio get");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            if (NRF_GPIO_PIN_DIR_OUTPUT == nrf_gpio_pin_dir_get((atoi(argv[1])))) {
                if (nrf_gpio_pin_out_read(atoi(argv[1]))) {
                    nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "The GPIO number %d connected\r\n", atoi(argv[1]));
                } else {
                    nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "The GPIO number %d disconnected\r\n", atoi(argv[1]));
                }
            } else {
                if (nrf_gpio_pin_read(atoi(argv[1]))) {
                    nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "The GPIO number %d connected\r\n", atoi(argv[1]));
                } else {
                    nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "The GPIO number %d disconnected\r\n", atoi(argv[1]));
                }
            }
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd_set(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio set");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            if (NRF_GPIO_PIN_DIR_OUTPUT == nrf_gpio_pin_dir_get((atoi(argv[1])))) {
                nrf_gpio_pin_set((atoi(argv[1])));
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "GPIO number %d set\r\n", atoi(argv[1]));
            } else {
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Canot set. Gpio pin number %d is not config as ouput.\r\n", atoi(argv[1]));
            }
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc == 1) {
        cli_missing_param_help(p_cli, "gpio");
    } else {
        cli_unknown_param_help(p_cli, argv[1], "gpio");
    }
}



static void gpio_cmd_reset(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio reset");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            if (NRF_GPIO_PIN_DIR_OUTPUT == nrf_gpio_pin_dir_get((atoi(argv[1])))) {
                nrf_gpio_pin_clear((atoi(argv[1])));
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "GPIO number %d reset\r\n", atoi(argv[1]));
            } else {
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Canot reset. Gpio pin number %d is not config as ouput.\r\n", atoi(argv[1]));
            }
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd_cfg_read(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio dir");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            if (nrf_gpio_pin_dir_get((atoi(argv[1]))) == NRF_GPIO_PIN_DIR_OUTPUT) {
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "GPIO number %d direction output\r\n", atoi(argv[1]));
            } else {
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "GPIO number %d direction input\r\n", atoi(argv[1]));
            }
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd_in_cfg(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio in_cfg");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            nrf_gpio_cfg_input(atoi(argv[1]), NRF_GPIO_PIN_NOPULL );
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Configuring the given GPIO pin number %d as input\r\n", atoi(argv[1]));
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd_out_cfg(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio out_cfg");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            nrf_gpio_cfg_output(atoi(argv[1]));
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Configuring the given GPIO pin number %d as output\r\n", atoi(argv[1]));
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}

static void gpio_cmd_toggle(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 2) {
        cli_unknown_param_help(p_cli, argv[1], "gpio toggle");
    } else {
        if (atoi(argv[1]) > 0 && atoi(argv[1]) < MAX_GPIO_NUM) {
            if (NRF_GPIO_PIN_DIR_OUTPUT == nrf_gpio_pin_dir_get((atoi(argv[1])))) {
                nrf_gpio_pin_toggle((atoi(argv[1])));
            } else {
                nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Canot toggle. Gpio pin number %d is not config as ouput.\r\n", atoi(argv[1]));
            }
        } else {
            nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "Error! please chooce gpio number between 0-%d.\r\n", MAX_GPIO_NUM);
        }
    }
}



uint32_t platform_read_ob_temp() {
    volatile int32_t temp;

    if (nrf_sdh_is_enabled()) {
        sd_temp_get((int32_t *)&temp);
        return temp / 4;
    }
 
    return 0;
}

static void temperature_cmd(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc == 1) {
        cli_missing_param_help(p_cli, "set");
    } else {
        cli_unknown_param_help(p_cli, argv[1], "set");
    }
}

static void temperature_cmd_read(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 1) {
        cli_unknown_param_help(p_cli, argv[1], "temperature read");
    } else {
        nrf_cli_fprintf(p_cli, NRF_CLI_NORMAL, "%d\r\n", platform_read_ob_temp());
    }
}
        

static void ble_cmd(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc == 1) {
        cli_missing_param_help(p_cli, "ble");
    } else {
        cli_unknown_param_help(p_cli, argv[1], "ble");
    }
}




NRF_CLI_CREATE_STATIC_SUBCMD_SET(m_gpio_cmd){
    NRF_CLI_CMD(get, NULL, "read gpio value\r\n", gpio_cmd_get),
    NRF_CLI_CMD(set, NULL, "set gpio to logic '1'\r\n", gpio_cmd_set),
    NRF_CLI_CMD(reset, NULL, "reset gpio to logic '0'\r\n", gpio_cmd_reset),
    NRF_CLI_CMD(cfg_read, NULL, "read direction of gpio\r\n", gpio_cmd_cfg_read),
    NRF_CLI_CMD(in_cfg, NULL, "set gpio as input\r\n", gpio_cmd_in_cfg),
    NRF_CLI_CMD(out_cfg, NULL, "set gpio as output\r\n", gpio_cmd_out_cfg),
    NRF_CLI_CMD(toggle, NULL, "change gpio value 0->1, 1->0\r\n", gpio_cmd_toggle),
    NRF_CLI_SUBCMD_SET_END};

NRF_CLI_CREATE_STATIC_SUBCMD_SET(m_temperature_cmd){
    NRF_CLI_CMD(read, NULL, "read on-die temperature sensor\r\n", temperature_cmd_read),
    NRF_CLI_SUBCMD_SET_END};


NRF_CLI_CMD_REGISTER(gpio, &m_gpio_cmd, GPIO_HELP, gpio_cmd);

NRF_CLI_CMD_REGISTER(temperature, &m_temperature_cmd, "interface with on-die temperature sensor\r\n", temperature_cmd);




static void reset_cmd(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 1) {
        cli_unknown_param_help(p_cli, argv[1], "reset");
    } else {
        nrf_pwr_mgmt_shutdown(NRF_PWR_MGMT_SHUTDOWN_RESET);
    }
}
NRF_CLI_CMD_REGISTER(reset, NULL, "reset\r\n", reset_cmd);

static void power_off_cmd(nrf_cli_t const *p_cli, size_t argc, char **argv) {
    if (nrf_cli_help_requested(p_cli)) {
        nrf_cli_help_print(p_cli, NULL, 0);
    } else if (argc != 1) {
        cli_unknown_param_help(p_cli, argv[1], "gpiot power_off");
    } else {
        nrf_pwr_mgmt_shutdown(NRF_PWR_MGMT_SHUTDOWN_STAY_IN_SYSOFF);
    }
}
NRF_CLI_CMD_REGISTER(power_off, NULL, "power off\r\n", power_off_cmd);


#endif